<?php

namespace App\Tests\Service;

/*
 * Copyright (C) 2020 Daniel Siepmann <coding@daniel-siepmann.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

use App\Entity\Entry;
use App\Persistence\Entries;
use App\Service\ActiveEntry;
use PHPUnit\Framework\TestCase;
use Prophecy\Argument;
use Symfony\Component\HttpFoundation\Session\SessionInterface;



/**
 * @covers App\Service\ActiveEntry
 */
class ActiveEntryTest extends TestCase
{
    /**
     * @test
     */
    public function canBeCreated(): void
    {
        $session = $this->prophesize(SessionInterface::class);
        $persistence = $this->prophesize(Entries::class);
        $subject = new ActiveEntry(
            $session->reveal(),
            $persistence->reveal()
        );

        static::assertInstanceOf(ActiveEntry::class, $subject);
    }

    /**
     * @test
     */
    public function returnsEmptyEntryAsDefault(): void
    {
        $session = $this->prophesize(SessionInterface::class);
        $persistence = $this->prophesize(Entries::class);
        $subject = new ActiveEntry(
            $session->reveal(),
            $persistence->reveal()
        );

        $entry = $subject->get();

        static::assertInstanceOf(Entry::class, $entry);
        static::assertSame('', $entry->getTitle());
        static::assertFalse($entry->isRunning());
    }

    /**
     * @test
     */
    public function canStartANewEntry(): void
    {
        $session = $this->prophesize(SessionInterface::class);
        $persistence = $this->prophesize(Entries::class);
        $subject = new ActiveEntry(
            $session->reveal(),
            $persistence->reveal()
        );

        $subject->startNew('Example title');

        $session->set('runningEntry', Argument::that(function (Entry $entry) {
            return $entry->getTitle() === 'Example title'
                && $entry->isRunning()
                ;
        }))->shouldBeCalled();
    }

    /**
     * @test
     */
    public function returnsStartedEntry(): void
    {
        $session = $this->prophesize(SessionInterface::class);
        $persistence = $this->prophesize(Entries::class);
        $entry = $this->prophesize(Entry::class);
        $subject = new ActiveEntry(
            $session->reveal(),
            $persistence->reveal()
        );

        $session->set('runningEntry', Argument::type(Entry::class))->shouldBeCalled();
        $session->get('runningEntry')->willReturn($entry->reveal())->shouldBeCalled();

        $subject->startNew('Example title');

        static::assertSame($entry->reveal(), $subject->get());
    }

    /**
     * @test
     */
    public function stopsRunningEntry(): void
    {
        $session = $this->prophesize(SessionInterface::class);
        $persistence = $this->prophesize(Entries::class);
        $entry = $this->prophesize(Entry::class);
        $subject = new ActiveEntry(
            $session->reveal(),
            $persistence->reveal()
        );

        $session->set('runningEntry', Argument::type(Entry::class))->shouldBeCalled();
        $session->get('runningEntry')->willReturn($entry->reveal())->shouldBeCalled();

        $subject->startNew('Example title');
        $subject->stopRunning();

        $session->remove('runningEntry')->shouldBeCalled();
        $persistence->add($entry->reveal())->shouldBeCalled();
    }
}
